<?php

namespace Drupal\media_folders\Form;

use Drupal\Component\Utility\Bytes;
use Drupal\Component\Utility\Environment;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Render\RendererInterface;
use Drupal\media_folders\MediaFoldersUiActions;
use Drupal\media_folders\MediaFoldersUiBuilder;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManager;

/**
 * Form builder.
 */
class AddWidgetFileForm extends FormBase {

  /**
   * The Get EntityTypeManagerInterface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The bundle info.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfo
   */
  protected $bundleInfo;

  /**
   * Drupal\Core\Render\RendererInterface definition.
   *
   * @var Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The folders ui actions service.
   *
   * @var \Drupal\media_folders\MediaFoldersUiActions
   */
  protected $foldersActionsUi;

  /**
   * The folders ui service.
   *
   * @var \Drupal\media_folders\MediaFoldersUiBuilder
   */
  protected $foldersUi;

  /**
   * The config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $fieldManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $current_user,
    EntityTypeBundleInfo $bundle_info,
    RendererInterface $renderer,
    MediaFoldersUiActions $media_folders_actions_ui,
    MediaFoldersUiBuilder $media_folders_ui,
    ConfigFactoryInterface $config_factory,
    EntityFieldManager $field_manager,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $this->entityTypeManager->getStorage('user')->load($current_user->id());
    $this->bundleInfo = $bundle_info;
    $this->renderer = $renderer;
    $this->foldersActionsUi = $media_folders_actions_ui;
    $this->foldersUi = $media_folders_ui;
    $this->configFactory = $config_factory;
    $this->fieldManager = $field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('entity_type.bundle.info'),
      $container->get('renderer'),
      $container->get('media_folders.ui_actions'),
      $container->get('media_folders.ui_builder'),
      $container->get('config.factory'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'add-widget-folder-form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, mixed $folder = NULL, $entity_type_id = NULL, $bundle = NULL, $widget_id = NULL) {
    if ($form_state->has('undefined_bundles') && !empty($form_state->get('undefined_bundles'))) {
      return $this->buildBundlesForm($form, $form_state);
    }

    $form_state->set('undefined_bundles', []);
    $field_validators = [
      'FileExtension' => [
        'extensions' => [],
      ],
      'FileSizeLimit' => [
        'fileLimit' => Environment::getUploadMaxSize(),
      ],
    ];

    $bundle_fields = $this->fieldManager->getFieldDefinitions($entity_type_id, $bundle);
    $settings = $bundle_fields[$widget_id]->getSettings();
    $target_bundles = array_values($settings['handler_settings']['target_bundles']);
    $form_state->set('target_bundles', $target_bundles);
    $validators = $this->foldersActionsUi->getFileValidators();
    $bundles = $this->bundleInfo->getBundleInfo('media');

    foreach (array_keys($bundles) as $bundle_id) {
      if (!in_array($bundle_id, $target_bundles)) {
        continue;
      }
      if ($this->foldersUi->hasMediaCreateAccess(NULL, $bundle_id, TRUE)) {
        if (!empty($validators[$bundle_id]['FileExtension']['extensions'])) {
          $field_validators['FileExtension']['extensions'][] = $validators[$bundle_id]['FileExtension']['extensions'];
          if (!empty($validators[$bundle_id]['FileSizeLimit']['fileLimit'])) {
            $bytes = Bytes::toNumber($validators[$bundle_id]['FileSizeLimit']['fileLimit']);
            if ($bytes < $field_validators['FileSizeLimit']['fileLimit']) {
              $field_validators['FileSizeLimit']['fileLimit'] = $bytes;
            }
          }
        }
      }
    }
    $field_validators['FileExtension']['extensions'] = implode(' ', $field_validators['FileExtension']['extensions']);

    $file_upload_help = [
      '#theme' => 'file_upload_help',
      '#upload_validators' => $field_validators,
    ];
    $form['file'] = [
      '#type' => 'managed_file',
      '#required' => TRUE,
      '#title' => $this->t('File'),
      '#multiple' => TRUE,
      '#default_value' => '',
      "#upload_location" => "temporary://media_folders",
      '#upload_validators' => $field_validators,
      '#description' => $this->renderer->renderInIsolation($file_upload_help),
    ];

    $form['parent'] = [
      '#type' => 'hidden',
      '#value' => ($folder) ? $folder : 0,
    ];

    $form['upload'] = [
      '#type' => 'button',
      '#ajax' => [
        'callback' => '::submitCallback',
        'wrapper' => 'upload-form',
      ],
      '#value' => $this->t('Upload files'),
    ];

    $form['upload'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#value' => $this->t('Upload files'),
      '#name' => 'next',
      '#submit' => ['::submitFormCallback'],
      '#ajax' => [
        'callback' => '::submitCallback',
        'event' => 'click',
        'progress' => [
          'type' => 'fullScreen',
        ],
      ],
    ];

    $form['#attributes']['id'] = 'form-content';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function buildBundlesForm(array $form, FormStateInterface $form_state) {
    $form['bundles'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Choose the bundle for the following files'),
    ];
    foreach ($form_state->get('undefined_bundles') as $key => $bundles) {
      $media_file = $this->entityTypeManager->getStorage('file')->load($key);
      $form['bundles'][$key] = [
        '#type' => 'select',
        '#options' => $bundles,
        '#title' => $this->t('Bundle for @filename', ['@filename' => $media_file->filename->value]),
      ];
    }

    $form['upload'] = [
      '#type' => 'button',
      '#ajax' => [
        'callback' => '::submitUploadCallback',
        'wrapper' => 'upload-form',
      ],
      '#value' => $this->t('Upload files'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
    if (!empty($form_state->get('possible_bundles'))) {
      $folder = $this->entityTypeManager->getStorage('taxonomy_term')->load($form_state->get('parent'));
      foreach ($form_state->get('possible_bundles') as $key => $values) {
        $this->foldersActionsUi->fileNameRename($values['media_file_name'], $form_state->get('filenames'));
        $bundle = (!empty($form_values[$key]) && !empty($values['bundles'][$form_values[$key]])) ? $form_values[$key] : key($values['bundles']);
        $this->foldersActionsUi->createMedia($values['media_file'], $folder, $bundle, $values['media_file_name']);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitUploadCallback(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $this->submitForm($form, $form_state);
    $response->addCommand(new MessageCommand($this->t('Files uploaded'), '#folders-messages'));
    $response->addCommand(new CloseDialogCommand('.ui-dialog:not([aria-modal="true"]) .ui-dialog-content'));
    $response->addCommand(new InvokeCommand('.navbar-folder[data-id=' . $form_state->get('parent') . '] a', 'click'));

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function submitFormCallback(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if (!empty($values['file']) && isset($values['parent'])) {
      $possible_bundles = [];
      $form_state->set('parent', $values['parent']);
      $filenames = $this->foldersActionsUi->getFolderFileNames($values['parent']);
      $form_state->set('filenames', $filenames);
      $bundles_labels = $this->bundleInfo->getBundleInfo('media');
      $target_bundles = $form_state->get('target_bundles');
      foreach ($values['file'] as $file_id) {
        if (is_numeric($file_id)) {
          $media_file = $this->entityTypeManager->getStorage('file')->load($file_id);
          $media_file_name = $media_file->filename->value;
          $possible_bundles[$file_id]['media_file'] = $media_file;
          $possible_bundles[$file_id]['media_file_name'] = $media_file_name;
          $position = strrpos($media_file_name, '.');
          $extension = substr($media_file_name, $position);
          $validators = $this->foldersActionsUi->getFileValidators();
          $bundles = $this->foldersActionsUi->findBundleByExtension($extension, $validators);
          foreach ($bundles as $bundle) {
            if ($bundle && in_array($bundle, $target_bundles) && $this->foldersUi->hasMediaCreateAccess(NULL, $bundle, TRUE)) {
              $possible_bundles[$file_id]['bundles'][$bundle] = $bundles_labels[$bundle]['label'];
            }
          }
        }
      }
      $form_state->set('possible_bundles', $possible_bundles);
      $undefined_bundles = [];
      foreach ($possible_bundles as $file_id => $bundles) {
        if (count($bundles['bundles']) > 1) {
          $undefined_bundles[$file_id] = $bundles['bundles'];
        }
      }
      if (!empty($undefined_bundles)) {
        $form_state->set('undefined_bundles', $undefined_bundles);
        $form_state->setValue('file', $values['file']);
        $form_state->setValue('parent', $values['parent']);
        $form_state->setRebuild();
      }
      else {
        $this->submitForm($form, $form_state);
        $form_state->set('success', TRUE);
        $form_state->setRebuild();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitCallback(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    if ($form_state->get('success')) {
      $response->addCommand(new MessageCommand($this->t('Files uploaded'), '#folders-messages'));
      $response->addCommand(new CloseDialogCommand('.ui-dialog:not([aria-modal="true"]) .ui-dialog-content'));
      $response->addCommand(new InvokeCommand('.navbar-folder[data-id=' . $form_state->get('parent') . '] a', 'click'));
    }
    else {
      if ($form_state->hasAnyErrors()) {
        $messages = $this->messenger()->deleteAll();
        $form['messages']['content'] = [
          '#theme' => 'status_messages',
          '#message_list' => $messages,
        ];
      }
      $response->addCommand(new ReplaceCommand('#form-content', $form));
    }

    return $response;
  }

}
